#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:s:"

/*
 * if not supplied, how many seconds to wait for event
 * and the minimum time we will allow to request
 * (since we use sleep() with a second granularity to
 * accuate the event)
 */
#define DEFAULT_WAIT_SEC    10
#define MIN_WAIT_SEC        2

/* 
 * for ultra sparc .. appears that it's
 * 100 ticks per second in default configuration
 */
#define TICKS_PER_SEC       100   

typedef struct
{
    TPRO_BoardObj*  hnd;
    int             secToEvent;
} ThreadInfo_t;

DWORD WINAPI WorkerThread(LPVOID args)
{
    int             rc;
    ThreadInfo_t*   info = (ThreadInfo_t*)args;

    Sleep (info->secToEvent);
    if ( (rc = TPRO_simEvent (info->hnd)) != TPRO_SUCCESS )
    {
        (void) printf("\n Error simulated time-tag event!\n" );
    }

    return 0;
}


/********************************************************
**
** Test Module: WaitEvent.c
**
** Purpose:     Wait for a simulated time-tag  event 
**              interrupt using a parameter-supplied number 
**              of seconds for the timeout value. 
**              The timeout is enforced to be no shorter 
**              than 2 seconds.  Whatever the timeout,
**              the event will be generated halfway through
**              the timeout requested.
**
*********************************************************/

int main (int argc, char *argv[])
{
    TPRO_BoardObj   *hnd = NULL;
    TPRO_WaitObj    WaitObj;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[MAX_DEVICE_PATH_LENGTH];
    char            *errMsg;
    char            *pgmname = argv[0];
    int             secToWait = DEFAULT_WAIT_SEC;
    int             secToEvent = 0;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 's' :
                secToWait = atoi(optarg);
                if ( secToWait < MIN_WAIT_SEC )
                {
                    usage( pgmname );
                    return (1);
                }
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * calc the time use to sleep before issuing the
     * simulated event.
     */
    secToEvent = (secToWait / 2) ;

    /*
     * construct the device name
     */
    (void)sprintf_s( fullDevName, MAX_DEVICE_PATH_LENGTH, "%s%s%d", DEVICE_ROOT,
		devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TPRO_open(&hnd, fullDevName)) != TPRO_SUCCESS )
    {
        /* 
         * if errors, close it
         */
        printf("%s: Could not open <%s> : rc <%d>\n", 
                        pgmname, fullDevName, rc);
        return (1);
    }

    DWORD threadId;
    ThreadInfo_t info;

    info.hnd = hnd;
    info.secToEvent = secToEvent;

    HANDLE hWorkerThread = CreateThread( NULL, 0, WorkerThread, &info, 0, &threadId);

    //error
    if(hWorkerThread == NULL)
    {
            ret = 1;
            (void) printf("\n%s: <%s> Unable to spawn process!\n", 
                                        pgmname, fullDevName );
    }
    else
    {
        /* 
         * wait for external time-tag event
         */
        (void) printf ("<%s> Waiting for time-tag event...", fullDevName);
        fflush (stdout);

        WaitObj.ticks = (secToWait * TICKS_PER_SEC) ;

        if ( (rc = TPRO_waitEvent(hnd, &WaitObj)) != TPRO_SUCCESS )
        {
            char emsg[MAX_ERR_MESG_LENGTH];

            switch (rc)
            {
                case TPRO_TIMEOUT_ERR:
                    (void)sprintf_s(emsg, MAX_ERR_MESG_LENGTH, " TIMEOUT_ERROR <%d> sec ", secToWait );
                    errMsg = emsg;
                    break;
                default:
					strcpy(errMsg, tpro_strerror((tpro_error_code)rc));
					break;
            }

            (void) printf("\n%s: <%s> %s rc: <%d>.\n", 
                        pgmname, fullDevName, errMsg, rc);

            ret = 1;
        }
        else
        {
            (void)printf(" received Days <%d> Hours <%d> Min <%d> Sec <%f>\n",
                            WaitObj.days, WaitObj.hours, 
                            WaitObj.minutes, WaitObj.seconds);
        }
    }

    /*
     * close the handle to the board
     */
    if ( (rc = TPRO_close(hnd)) != TPRO_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n", 
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-s SECONDS] [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-s   # of sec to wait for event (dflt %d,  min <%d>)\n", 
                                        DEFAULT_WAIT_SEC, MIN_WAIT_SEC );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n\n", DEFAULT_DEV);
}

